﻿using System;
using System.Net.Security;
using System.ServiceModel;
using System.Xml.Serialization;

namespace Nardax.ServiceModel.Discovery.Bindings
{
    [XmlRoot(BindingSerializer.RootElementName)]
    public class SerializableNetTcpBinding : SerializableBinding<NetTcpBinding>
    {
        public SerializableNetTcpBinding()
        {
            Binding = new NetTcpBinding();
        }

        public SerializableNetTcpBinding(NetTcpBinding binding)
        {
            Binding = binding;
        }

        [XmlAttribute("HNCM")]
        public HostNameComparisonMode HostNameComparisonMode
        {
            get { return Binding.HostNameComparisonMode; }
            set { Binding.HostNameComparisonMode = value; }
        }

        [XmlAttribute("LB")]
        public int ListenBacklog
        {
            get { return Binding.ListenBacklog; }
            set { Binding.ListenBacklog = value; }
        }

        [XmlAttribute("MBPS")]
        public long MaxBufferPoolSize
        {
            get { return Binding.MaxBufferPoolSize; }
            set { Binding.MaxBufferPoolSize = value; }
        }

        [XmlAttribute("MBS")]
        public int MaxBufferSize
        {
            get { return Binding.MaxBufferSize; }
            set { Binding.MaxBufferSize = value; }
        }

        [XmlAttribute("MC")]
        public int MaxConnections
        {
            get { return Binding.MaxConnections; }
            set { Binding.MaxConnections = value; }
        }

        [XmlAttribute("MRMS")]
        public long MaxReceivedMessageSize
        {
            get { return Binding.MaxReceivedMessageSize; }
            set { Binding.MaxReceivedMessageSize = value; }
        }

        [XmlAttribute("PSE")]
        public bool PortSharingEnabled
        {
            get { return Binding.PortSharingEnabled; }
            set { Binding.PortSharingEnabled = value; }
        }

        [XmlAttribute("TF")]
        public bool TransactionFlow
        {
            get { return Binding.TransactionFlow; }
            set { Binding.TransactionFlow = value; }
        }

        [XmlAttribute("TP")]
        public string TransactionProtocol
        {
            get { return Binding.TransactionProtocol.GetType().Name; }
            set
            {
                switch (value)
                {
                    case "OleTransactionsProtocol":
                        Binding.TransactionProtocol = System.ServiceModel.TransactionProtocol.OleTransactions;
                        break;
                    case "WSAtomicTransaction11Protocol":
                        Binding.TransactionProtocol = System.ServiceModel.TransactionProtocol.WSAtomicTransaction11;
                        break;
                    case "WSAtomicTransactionOctober2004Protocol":
                        Binding.TransactionProtocol = System.ServiceModel.TransactionProtocol.WSAtomicTransactionOctober2004;
                        break;
                }
            }
        }

        [XmlAttribute("TM")]
        public TransferMode TransferMode
        {
            get { return Binding.TransferMode; }
            set { Binding.TransferMode = value; }
        }

        [XmlAttribute("RQ-MAL")]
        public int MaxArrayLength
        {
            get { return Binding.ReaderQuotas.MaxArrayLength; }
            set { Binding.ReaderQuotas.MaxArrayLength = value; }
        }

        [XmlAttribute("RQ-MBPR")]
        public int MaxBytesPerRead
        {
            get { return Binding.ReaderQuotas.MaxBytesPerRead; }
            set { Binding.ReaderQuotas.MaxBytesPerRead = value; }
        }

        [XmlAttribute("RQ-MD")]
        public int MaxDepth
        {
            get { return Binding.ReaderQuotas.MaxDepth; }
            set { Binding.ReaderQuotas.MaxDepth = value; }
        }

        [XmlAttribute("RQ-MNTCC")]
        public int MaxNameTableCharCount
        {
            get { return Binding.ReaderQuotas.MaxNameTableCharCount; }
            set { Binding.ReaderQuotas.MaxNameTableCharCount = value; }
        }

        [XmlAttribute("RQ-MSCL")]
        public int MaxStringContentLength
        {
            get { return Binding.ReaderQuotas.MaxStringContentLength; }
            set { Binding.ReaderQuotas.MaxStringContentLength = value; }
        }

        [XmlAttribute("RS-E")]
        public bool Enabled
        {
            get { return Binding.ReliableSession.Enabled; }
            set { Binding.ReliableSession.Enabled = value; }
        }

        [XmlAttribute("RS-IT")]
        public string InactivityTimeout
        {
            get { return Binding.ReliableSession.InactivityTimeout.ToString(); }
            set { Binding.ReliableSession.InactivityTimeout = TimeSpan.Parse(value); }
        }

        [XmlAttribute("RS-O")]
        public bool Ordered
        {
            get { return Binding.ReliableSession.Ordered; }
            set { Binding.ReliableSession.Ordered = value; }
        }

        [XmlAttribute("S-M")]
        public SecurityMode SecurityMode
        {
            get { return Binding.Security.Mode; }
            set { Binding.Security.Mode = value; }
        }

        [XmlAttribute("S-M-AS")]
        public string SecurityAlgorithmSuite
        {
            get { return Binding.Security.Message.AlgorithmSuite.GetType().Name; }
            set
            {
                switch (value)
                {
                    case "Basic256SecurityAlgorithmSuite":
                        Binding.Security.Message.AlgorithmSuite = System.ServiceModel.Security.SecurityAlgorithmSuite.Basic256;
                        break;

                    case "Basic192SecurityAlgorithmSuite":
                        Binding.Security.Message.AlgorithmSuite = System.ServiceModel.Security.SecurityAlgorithmSuite.Basic192;
                        break;

                    case "Basic128SecurityAlgorithmSuite":
                        Binding.Security.Message.AlgorithmSuite = System.ServiceModel.Security.SecurityAlgorithmSuite.Basic128;
                        break;

                    case "TripleDesSecurityAlgorithmSuite":
                        Binding.Security.Message.AlgorithmSuite = System.ServiceModel.Security.SecurityAlgorithmSuite.TripleDes;
                        break;

                    case "Basic256Rsa15SecurityAlgorithmSuite":
                        Binding.Security.Message.AlgorithmSuite = System.ServiceModel.Security.SecurityAlgorithmSuite.Basic256Rsa15;
                        break;

                    case "Basic192Rsa15SecurityAlgorithmSuite":
                        Binding.Security.Message.AlgorithmSuite = System.ServiceModel.Security.SecurityAlgorithmSuite.Basic192Rsa15;
                        break;

                    case "Basic128Rsa15SecurityAlgorithmSuite":
                        Binding.Security.Message.AlgorithmSuite = System.ServiceModel.Security.SecurityAlgorithmSuite.Basic128Rsa15;
                        break;

                    case "TripleDesRsa15SecurityAlgorithmSuite":
                        Binding.Security.Message.AlgorithmSuite = System.ServiceModel.Security.SecurityAlgorithmSuite.TripleDesRsa15;
                        break;

                    case "Basic256Sha256SecurityAlgorithmSuite":
                        Binding.Security.Message.AlgorithmSuite = System.ServiceModel.Security.SecurityAlgorithmSuite.Basic256Sha256;
                        break;

                    case "Basic192Sha256SecurityAlgorithmSuite":
                        Binding.Security.Message.AlgorithmSuite = System.ServiceModel.Security.SecurityAlgorithmSuite.Basic192Sha256;
                        break;

                    case "Basic128Sha256SecurityAlgorithmSuite":
                        Binding.Security.Message.AlgorithmSuite = System.ServiceModel.Security.SecurityAlgorithmSuite.Basic128Sha256;
                        break;

                    case "TripleDesSha256SecurityAlgorithmSuite":
                        Binding.Security.Message.AlgorithmSuite = System.ServiceModel.Security.SecurityAlgorithmSuite.TripleDesSha256;
                        break;

                    case "Basic256Sha256Rsa15SecurityAlgorithmSuite":
                        Binding.Security.Message.AlgorithmSuite = System.ServiceModel.Security.SecurityAlgorithmSuite.Basic256Sha256Rsa15;
                        break;

                    case "Basic192Sha256Rsa15SecurityAlgorithmSuite":
                        Binding.Security.Message.AlgorithmSuite = System.ServiceModel.Security.SecurityAlgorithmSuite.Basic192Sha256Rsa15;
                        break;

                    case "Basic128Sha256Rsa15SecurityAlgorithmSuite":
                        Binding.Security.Message.AlgorithmSuite = System.ServiceModel.Security.SecurityAlgorithmSuite.Basic128Sha256Rsa15;
                        break;

                    case "TripleDesSha256Rsa15SecurityAlgorithmSuite":
                        Binding.Security.Message.AlgorithmSuite = System.ServiceModel.Security.SecurityAlgorithmSuite.TripleDesSha256Rsa15;
                        break;
                }
            }
        }

        [XmlAttribute("S-M-CCT")]
        public MessageCredentialType MessageClientCredentialType
        {
            get { return Binding.Security.Message.ClientCredentialType; }
            set { Binding.Security.Message.ClientCredentialType = value; }
        }

        [XmlAttribute("S-T-PL")]
        public ProtectionLevel ProtectionLevel
        {
            get { return Binding.Security.Transport.ProtectionLevel; }
            set { Binding.Security.Transport.ProtectionLevel = value; }
        }

        [XmlAttribute("S-T-CCT")]
        public TcpClientCredentialType TransportClientCredentialType
        {
            get { return Binding.Security.Transport.ClientCredentialType; }
            set { Binding.Security.Transport.ClientCredentialType = value; }
        }
    }
}
