﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Data.SqlClient;
using Nardax.Data;  

namespace Nardax.Data.Tests
{
    [TestClass]
    public class IDataParameterExtensionsTests
    {
        [TestMethod]
        public void GetParameterValueAsGuid_ReturnsGuid()
        {
            var expected = Guid.NewGuid();
            var parameter = new SqlParameter("@Guid", expected);
            var actual = parameter.GetValue<Guid>();

            Assert.AreEqual(expected, actual);            
        }

        [TestMethod]
        public void GetParameterValueAsNullableGuid_ReturnsGuidAsNullableBuid()
        {
            Guid? expected = Guid.NewGuid();
            var parameter = new SqlParameter("@Guid", expected);
            var actual = parameter.GetNullableValue<Guid>();

            Assert.AreEqual(expected, actual); 
        }

        [TestMethod]
        public void GetParameterValueAsNullableGuid_ReturnsDBNullAsNullableGuid()
        {
            var parameter = new SqlParameter("@Guid", DBNull.Value);
            var actual = parameter.GetNullableValue<Guid>();

            Assert.IsFalse(actual.HasValue);
        }

        [TestMethod]
        public void GetParameterValueAsString_ReturnsString()
        {
            var expected = "TestString";
            var parameter = new SqlParameter("@String", expected);
            var actual = parameter.GetValue<String>();

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void GetParameterValueAsDBNull_ReturnsEmptyString()
        {
            var expected = String.Empty;
            var parameter = new SqlParameter("@String", DBNull.Value);
            var actual = parameter.GetValue<String>();

            Assert.AreEqual(expected, actual);
        }
        
        [TestMethod]
        public void GetParameterValueAsEnumWhenRetunedAsInt()
        {
            var parameterValue = 2;
            var parameter = new SqlParameter("@String", parameterValue);
            var actual = parameter.GetValue<TestEnum>();

            var expected = TestEnum.Code;
            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void GetParamaterValueAsEnumWhenReturnedAsString()
        {
            var parameterValue = "Code";
            var parameter = new SqlParameter("@String", parameterValue);
            var actual = parameter.GetValue<TestEnum>();

            var expected = TestEnum.Code;
            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void GetParameterValueAsNullableEnumWhenReturnedAsDBNullValue()
        {
            var parameterValue = DBNull.Value;
            var parameter = new SqlParameter("@String", parameterValue);
            var actual = parameter.GetNullableValue<TestEnum>();

            TestEnum? expected = null;
            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void GetParameterValueAsNullableEnumWhenReturnedAsInt()
        {
            var parameterValue = 2;
            var parameter = new SqlParameter("@String", parameterValue);
            var actual = parameter.GetNullableValue<TestEnum>();

            TestEnum? expected = TestEnum.Code;
            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void GetParameterValueAsNullableEnumWhenReturnedAsString()
        {
            var parameterValue = "Code";
            var parameter = new SqlParameter("@String", parameterValue);
            var actual = parameter.GetNullableValue<TestEnum>();

            TestEnum? expected = TestEnum.Code;
            Assert.AreEqual(expected, actual);
        }
    }
}
