﻿using Nardax.Data;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Data;
using System.Data.SqlClient;
using Rhino.Mocks;

namespace Nardax.Data.Tests
{
    [TestClass()]
    public class IDataReaderExtensionsTests
    {
        private IDataReader _reader;

        [TestInitialize()]
        public void Initialize()
        {
            _reader = MockRepository.GenerateStub<IDataReader>();
        }

        [TestMethod]
        public void GetEnumValueFromReaderWhenStoredAsInt()
        {
            var columnName = "EnumValue";
            var columnValue = 1;

            _reader.Stub(x => x[columnName]).Return(columnValue);

            var actual = _reader.GetValue<TestEnum>(columnName);

            var expected = TestEnum.Sample;
            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void GetEnumValueFromReaderWhenStoredAsString()
        {
            var columnName = "EnumValue";
            var columnValue = "Sample";

            _reader.Stub(x => x[columnName]).Return(columnValue);

            var actual = _reader.GetValue<TestEnum>(columnName);

            var expected = TestEnum.Sample;
            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void GetNullableEnumValueFromReaderWhenColumnValueIsDBNull()
        {
            var columnName = "EnumValue";
            var columnValue = DBNull.Value;

            _reader.Stub(x => x[columnName]).Return(columnValue);

            var actual = _reader.GetNullableValue<TestEnum>(columnName);

            TestEnum? expected = null;
            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void GetNullableEnumValueFromReaderWhenStoredAsInt()
        {
            var columnName = "EnumValue";
            var columnValue = 1;

            _reader.Stub(x => x[columnName]).Return(columnValue);

            var actual = _reader.GetNullableValue<TestEnum>(columnName);

            TestEnum? expected = TestEnum.Sample;
            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void GetNullableEnumValueFromReaderWhenStoredAsString()
        {
            var columnName = "EnumValue";
            var columnValue = "Sample";

            _reader.Stub(x => x[columnName]).Return(columnValue);

            var actual = _reader.GetNullableValue<TestEnum>(columnName);

            TestEnum? expected = TestEnum.Sample;
            Assert.AreEqual(expected, actual);
        }

        [TestMethod()]
        public void GetInt32ValueFromReader_ReturnsInt32()
        {
            var columnName = "IntValue";
            var columnValue = 32;

            _reader.Stub(x => x[columnName]).Return(columnValue);

            var actual = _reader.GetValue<int>(columnName);

            Assert.AreEqual(columnValue, actual);
        }

        [TestMethod()]
        public void GetNullableInt32ValueFromReader_ReturnsNullableInt32()
        {
            var columnName = "IntValue";
            int? columnValue = 32;

            _reader.Stub(x => x[columnName]).Return(columnValue);

            var actual = _reader.GetNullableValue<int>(columnName);

            Assert.AreEqual(columnValue, actual);
        }

        [TestMethod()]
        public void GetNullableInt32ValueFromReader_ReturnsNullWhenValueIsDBNull()
        {
            var columnName = "IntValue";
            var columnValue = DBNull.Value;

            _reader.Stub(x => x[columnName]).Return(columnValue);

            var actual = _reader.GetNullableValue<int>(columnName);

            int? expected = null;
            Assert.AreEqual(expected, actual);
        }
    }
}
