﻿using System.Net.Security;
using System.ServiceModel;
using System.ServiceModel.Security;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Nardax.ServiceModel.Discovery.Bindings;

namespace Nardax.ServiceModel.Discovery.Tests
{
    [TestClass]
    public class SerializableNetTcpBindingTests
    {
        private SerializableNetTcpBinding _serializableBinding;

        [TestInitialize]
        public void MyTestInitialize()
        {
            _serializableBinding = new SerializableNetTcpBinding();
        }

        [TestMethod]
        public void ConstructorSettingsBinding()
        {
            var binding = new NetTcpBinding();
            var serializableBinding = new SerializableNetTcpBinding(binding);
            
            Assert.AreSame(binding, serializableBinding.Binding);
        }

        [TestMethod]
        public void GetTypeTest()
        {
            var expected = _serializableBinding.Binding.GetType().ToString();
            var actual = _serializableBinding.Type;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SetTypeTest()
        {
            _serializableBinding.Type = "This is not a type";
            var expected = _serializableBinding.Binding.GetType().ToString();
            var actual = _serializableBinding.Type;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void CloseTimeoutTest()
        {
            var expected = "00:01:00";
            _serializableBinding.CloseTimeout = expected;

            var actual = _serializableBinding.CloseTimeout;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void OpenTimeoutTest()
        {
            var expected = "00:02:00";
            _serializableBinding.OpenTimeout = expected;

            var actual = _serializableBinding.OpenTimeout;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void ReceiveTimeout()
        {
            var expected = "00:03:00";
            _serializableBinding.ReceiveTimeout = expected;

            var actual = _serializableBinding.ReceiveTimeout;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SendTimeout()
        {
            var expected = "00:04:00";
            _serializableBinding.SendTimeout = expected;

            var actual = _serializableBinding.SendTimeout;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void EnabledTest()
        {
            var expected = true;
            _serializableBinding.Enabled = expected;

            var actual = _serializableBinding.Enabled;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void HostNameComparisonModeTest()
        {
            var expected = HostNameComparisonMode.WeakWildcard;
            _serializableBinding.HostNameComparisonMode = expected;

            var actual = _serializableBinding.HostNameComparisonMode;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void InactivityTimeoutTest()
        {
            var expected = "00:00:01";
            _serializableBinding.InactivityTimeout = expected;

            var actual = _serializableBinding.InactivityTimeout;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void ListenBacklogTest()
        {
            var expected = 7;
            _serializableBinding.ListenBacklog = expected;

            var actual = _serializableBinding.ListenBacklog;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void MaxArrayLengthTest()
        {
            var expected = 7;
            _serializableBinding.MaxArrayLength = expected;

            var actual = _serializableBinding.MaxArrayLength;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void MaxBufferPoolSizeTest()
        {
            var expected = 1000L;
            _serializableBinding.MaxBufferPoolSize = expected;

            var actual = _serializableBinding.MaxBufferPoolSize;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void MaxBufferSizeTest()
        {
            var expected = 1000;
            _serializableBinding.MaxBufferSize = expected;

            var actual = _serializableBinding.MaxBufferSize;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void MaxBytesPerReadTest()
        {
            var expected = 3345;
            _serializableBinding.MaxBytesPerRead = expected;

            var actual = _serializableBinding.MaxBytesPerRead;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void MaxConnectionsTest()
        {
            var expected = 223;
            _serializableBinding.MaxConnections = expected;

            var actual = _serializableBinding.MaxConnections;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void MaxDepthTest()
        {
            var expected = 432;
            _serializableBinding.MaxDepth = expected;

            var actual = _serializableBinding.MaxDepth;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void MaxNameTableCharCountTest()
        {
            var expected = 48;
            _serializableBinding.MaxNameTableCharCount = expected;

            var actual = _serializableBinding.MaxNameTableCharCount;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void MaxReceivedMessageSizeTest()
        {
            var expected = 4898L;
            _serializableBinding.MaxReceivedMessageSize = expected;

            var actual = _serializableBinding.MaxReceivedMessageSize;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void MaxStringContentLengthTest()
        {
            var expected = 22;
            _serializableBinding.MaxStringContentLength = expected;

            var actual = _serializableBinding.MaxStringContentLength;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void MessageClientCredentialTypeTest()
        {
            var expected = MessageCredentialType.None;
            _serializableBinding.MessageClientCredentialType = expected;

            var actual = _serializableBinding.MessageClientCredentialType;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void OrderedTest()
        {
            var expected = true;
            _serializableBinding.Ordered = expected;

            var actual = _serializableBinding.Ordered;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void PortSharingEnabledTest()
        {
            var expected = true;
            _serializableBinding.PortSharingEnabled = expected;

            var actual = _serializableBinding.PortSharingEnabled;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void ProtectionLevelTest()
        {
            var expected = ProtectionLevel.EncryptAndSign;
            _serializableBinding.ProtectionLevel = expected;

            var actual = _serializableBinding.ProtectionLevel;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteBasic256Test()
        {
            // change the sercurity algorithmSuite first because Basic256 is the default
            _serializableBinding.SecurityAlgorithmSuite = SecurityAlgorithmSuite.Basic192.GetType().Name;

            var expected = SecurityAlgorithmSuite.Basic256.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteBasic192Test()
        {
            var expected = SecurityAlgorithmSuite.Basic192.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteBasic128Test()
        {
            var expected = SecurityAlgorithmSuite.Basic128.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteTripleDesTest()
        {
            var expected = SecurityAlgorithmSuite.TripleDes.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteBasic256Rsa15Test()
        {
            var expected = SecurityAlgorithmSuite.Basic256Rsa15.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteBasic192Rsa15Test()
        {
            var expected = SecurityAlgorithmSuite.Basic192Rsa15.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteBasic128Rsa15Test()
        {
            var expected = SecurityAlgorithmSuite.Basic128Rsa15.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteTripleDesRsa15Test()
        {
            var expected = SecurityAlgorithmSuite.TripleDesRsa15.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteBasic256Sha256Test()
        {
            var expected = SecurityAlgorithmSuite.Basic256Sha256.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteBasic192Sha256Test()
        {
            var expected = SecurityAlgorithmSuite.Basic192Sha256.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteBasic128Sha256Test()
        {
            var expected = SecurityAlgorithmSuite.Basic128Sha256.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteTripleDesSha256Test()
        {
            var expected = SecurityAlgorithmSuite.TripleDesSha256.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteBasic256Sha256Rsa15Test()
        {
            var expected = SecurityAlgorithmSuite.Basic256Sha256Rsa15.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteBasic192Sha256Rsa15Test()
        {
            var expected = SecurityAlgorithmSuite.Basic192Sha256Rsa15.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteBasic128Sha256Rsa15Test()
        {
            var expected = SecurityAlgorithmSuite.Basic128Sha256Rsa15.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteTripleDesSha256Rsa15Test()
        {
            var expected = SecurityAlgorithmSuite.TripleDesSha256Rsa15.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityModeTest()
        {
            var expected = SecurityMode.Transport;
            _serializableBinding.SecurityMode = expected;

            var actual = _serializableBinding.SecurityMode;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void TransactionFlowTest()
        {
            var expected = true;
            _serializableBinding.TransactionFlow = expected;

            var actual = _serializableBinding.TransactionFlow;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void TransactionProtocolOleTransactionsTest()
        {
            // change the transaction protocol once here because OleTransactions is the default
            _serializableBinding.TransactionProtocol = TransactionProtocol.WSAtomicTransaction11.GetType().Name;

            var expected = TransactionProtocol.OleTransactions.GetType().Name;
            _serializableBinding.TransactionProtocol = expected;

            var actual = _serializableBinding.TransactionProtocol;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void TransactionProtocolWSAtomicTransaction11Test()
        {
            var expected = TransactionProtocol.WSAtomicTransaction11.GetType().Name;
            _serializableBinding.TransactionProtocol = expected;

            var actual = _serializableBinding.TransactionProtocol;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void TransactionProtocolWSAtomicTransactionOctober2004Test()
        {
            var expected = TransactionProtocol.WSAtomicTransactionOctober2004.GetType().Name;
            _serializableBinding.TransactionProtocol = expected;

            var actual = _serializableBinding.TransactionProtocol;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void TransferModeTest()
        {
            var expected = TransferMode.Streamed;
            _serializableBinding.TransferMode = expected;

            var actual = _serializableBinding.TransferMode;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void TransportClientCredentialTypeTest()
        {
            var expected = TcpClientCredentialType.Windows;
            _serializableBinding.TransportClientCredentialType = expected;

            var actual = _serializableBinding.TransportClientCredentialType;

            Assert.AreEqual(expected, actual);
        }
    }
}