﻿using System.ServiceModel.Security;
using System.Text;
using Nardax.ServiceModel.Discovery.Bindings;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.ServiceModel;

namespace Nardax.ServiceModel.Discovery.Tests
{


    [TestClass()]
    public class SerializableWSHttpBindingTests
    {
        private SerializableWSHttpBinding _serializableBinding;

        [TestInitialize]
        public void MyTestInitialize()
        {
            _serializableBinding = new SerializableWSHttpBinding();
        }

        [TestMethod]
        public void ConstructorSettingsBinding()
        {
            var binding = new WSHttpBinding();
            var serializableBinding = new SerializableWSHttpBinding(binding);

            Assert.AreSame(binding, serializableBinding.Binding);
        }

        [TestMethod()]
        public void AllowCookiesTest()
        {
            var expected = true;
            _serializableBinding.AllowCookies = expected;

            var actual = _serializableBinding.AllowCookies;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod()]
        public void BypassProxyOnLocalTest()
        {
            var expected = true;
            _serializableBinding.BypassProxyOnLocal = expected;

            var actual = _serializableBinding.BypassProxyOnLocal;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void EnabledTest()
        {
            var expected = true;
            _serializableBinding.Enabled = expected;

            var actual = _serializableBinding.Enabled;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod()]
        public void EstablishSecurityContextTest()
        {
            var expected = true;
            _serializableBinding.EstablishSecurityContext = expected;

            var actual = _serializableBinding.EstablishSecurityContext;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void HostNameComparisonModeTest()
        {
            var expected = HostNameComparisonMode.WeakWildcard;
            _serializableBinding.HostNameComparisonMode = expected;

            var actual = _serializableBinding.HostNameComparisonMode;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void InactivityTimeoutTest()
        {
            var expected = "00:00:01";
            _serializableBinding.InactivityTimeout = expected;

            var actual = _serializableBinding.InactivityTimeout;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void MaxArrayLengthTest()
        {
            var expected = 7;
            _serializableBinding.MaxArrayLength = expected;

            var actual = _serializableBinding.MaxArrayLength;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void MaxBufferPoolSizeTest()
        {
            var expected = 1000L;
            _serializableBinding.MaxBufferPoolSize = expected;

            var actual = _serializableBinding.MaxBufferPoolSize;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod()]
        public void MaxBytesPerReadTest()
        {
            var expected = 1033;
            _serializableBinding.MaxBytesPerRead = expected;

            var actual = _serializableBinding.MaxBytesPerRead;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod()]
        public void MaxDepthTest()
        {
            var expected = 13;
            _serializableBinding.MaxDepth = expected;

            var actual = _serializableBinding.MaxDepth;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod()]
        public void MaxNameTableCharCountTest()
        {
            var expected = 323;
            _serializableBinding.MaxNameTableCharCount = expected;

            var actual = _serializableBinding.MaxNameTableCharCount;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void MaxReceivedMessageSizeTest()
        {
            var expected = 4898L;
            _serializableBinding.MaxReceivedMessageSize = expected;

            var actual = _serializableBinding.MaxReceivedMessageSize;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void MaxStringContentLengthTest()
        {
            var expected = 22;
            _serializableBinding.MaxStringContentLength = expected;

            var actual = _serializableBinding.MaxStringContentLength;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void MessageClientCredentialTypeTest()
        {
            var expected = MessageCredentialType.None;
            _serializableBinding.MessageClientCredentialType = expected;

            var actual = _serializableBinding.MessageClientCredentialType;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod()]
        public void MessageEncodingTest()
        {
            var expected = WSMessageEncoding.Mtom;
            _serializableBinding.MessageEncoding = expected;

            var actual = _serializableBinding.MessageEncoding;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod()]
        public void NegotiateServiceCredentialTest()
        {
            var expected = true;
            _serializableBinding.NegotiateServiceCredential = expected;

            var actual = _serializableBinding.NegotiateServiceCredential;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void OrderedTest()
        {
            var expected = true;
            _serializableBinding.Ordered = expected;

            var actual = _serializableBinding.Ordered;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod()]
        public void ProxyAddressTest()
        {
            var expected = "http://proxy/";
            _serializableBinding.ProxyAddress = expected;

            var actual = _serializableBinding.ProxyAddress;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod()]
        public void NullProxyAddressTest()
        {
            var expected = default(string);
            _serializableBinding.ProxyAddress = expected;

            var actual = _serializableBinding.ProxyAddress;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod()]
        public void ProxyCredentialTypeTest()
        {
            var expected = HttpProxyCredentialType.Digest;
            _serializableBinding.ProxyCredentialType = expected;

            var actual = _serializableBinding.ProxyCredentialType;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod()]
        public void RealmTest()
        {
            var expected = "fake";
            _serializableBinding.Realm = expected;

            var actual = _serializableBinding.Realm;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteBasic256Test()
        {
            // change the sercurity algorithmSuite first because Basic256 is the default
            _serializableBinding.SecurityAlgorithmSuite = SecurityAlgorithmSuite.Basic192.GetType().Name;

            var expected = SecurityAlgorithmSuite.Basic256.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteBasic192Test()
        {
            var expected = SecurityAlgorithmSuite.Basic192.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteBasic128Test()
        {
            var expected = SecurityAlgorithmSuite.Basic128.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteTripleDesTest()
        {
            var expected = SecurityAlgorithmSuite.TripleDes.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteBasic256Rsa15Test()
        {
            var expected = SecurityAlgorithmSuite.Basic256Rsa15.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteBasic192Rsa15Test()
        {
            var expected = SecurityAlgorithmSuite.Basic192Rsa15.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteBasic128Rsa15Test()
        {
            var expected = SecurityAlgorithmSuite.Basic128Rsa15.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteTripleDesRsa15Test()
        {
            var expected = SecurityAlgorithmSuite.TripleDesRsa15.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteBasic256Sha256Test()
        {
            var expected = SecurityAlgorithmSuite.Basic256Sha256.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteBasic192Sha256Test()
        {
            var expected = SecurityAlgorithmSuite.Basic192Sha256.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteBasic128Sha256Test()
        {
            var expected = SecurityAlgorithmSuite.Basic128Sha256.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteTripleDesSha256Test()
        {
            var expected = SecurityAlgorithmSuite.TripleDesSha256.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteBasic256Sha256Rsa15Test()
        {
            var expected = SecurityAlgorithmSuite.Basic256Sha256Rsa15.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteBasic192Sha256Rsa15Test()
        {
            var expected = SecurityAlgorithmSuite.Basic192Sha256Rsa15.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteBasic128Sha256Rsa15Test()
        {
            var expected = SecurityAlgorithmSuite.Basic128Sha256Rsa15.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityAlgorithmSuiteTripleDesSha256Rsa15Test()
        {
            var expected = SecurityAlgorithmSuite.TripleDesSha256Rsa15.GetType().Name;
            _serializableBinding.SecurityAlgorithmSuite = expected;

            var actual = _serializableBinding.SecurityAlgorithmSuite;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void SecurityModeTest()
        {
            var expected = SecurityMode.Transport;
            _serializableBinding.SecurityMode = expected;

            var actual = _serializableBinding.SecurityMode;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod()]
        public void TextEncodingUTF8Test()
        {
            // change the value first because UTF8 is the default
            _serializableBinding.TextEncoding = Encoding.Unicode.WebName;

            var expected = Encoding.UTF8.WebName;
            _serializableBinding.TextEncoding = expected;

            var actual = _serializableBinding.TextEncoding;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod()]
        public void TextEncodingUnicodeTest()
        {
            var expected = Encoding.Unicode.WebName;
            _serializableBinding.TextEncoding = expected;

            var actual = _serializableBinding.TextEncoding;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod()]
        public void TextEncodingBigEndianUnicodeTest()
        {
            var expected = Encoding.BigEndianUnicode.WebName;
            _serializableBinding.TextEncoding = expected;

            var actual = _serializableBinding.TextEncoding;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void TransactionFlowTest()
        {
            var expected = true;
            _serializableBinding.TransactionFlow = expected;

            var actual = _serializableBinding.TransactionFlow;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void TransportClientCredentialTypeTest()
        {
            var expected = HttpClientCredentialType.Certificate;
            _serializableBinding.TransportClientCredentialType = expected;

            var actual = _serializableBinding.TransportClientCredentialType;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod()]
        public void UseDefaultWebProxyTest()
        {
            var expected = true;
            _serializableBinding.UseDefaultWebProxy = expected;

            var actual = _serializableBinding.UseDefaultWebProxy;

            Assert.AreEqual(expected, actual);
        }
    }
}
